@@ -1,12 +1,12 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2006-2007, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2006-2010, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: SoftReference.scala 13755 2008-01-21 22:19:58Z odersky $
+// $Id: SoftReference.scala 20341 2010-01-04 12:06:07Z phaller $
 
 package scala.ref
 
@@ -15,7 +15,12 @@ package scala.ref
  */
 class SoftReference[+T <: AnyRef](value : T, queue : ReferenceQueue[T]) extends ReferenceWrapper[T] {
   def this(value : T) = this(value, null);
-  val underlying: java.lang.ref.SoftReference[_ <: T] = 
-    if (queue == null) new java.lang.ref.SoftReference[T](value);
-    else new java.lang.ref.SoftReference[T](value, queue.underlying.asInstanceOf[java.lang.ref.ReferenceQueue[T]])
+  val underlying: java.lang.ref.SoftReference[_ <: T] =
+    new SoftReferenceWithWrapper[T](value, queue, this)
 }
+
+/**
+ *  @author Philipp Haller
+ */
+private class SoftReferenceWithWrapper[T <: AnyRef](value: T, queue: ReferenceQueue[T], val wrapper: SoftReference[T])
+  extends java.lang.ref.SoftReference[T](value, if (queue == null) null else queue.underlying.asInstanceOf[java.lang.ref.ReferenceQueue[T]]) with ReferenceWithWrapper[T]
